var _excluded = ["strengthMultiplier", "verticalStrength", "horizontalStrength", "onScrollChange"];

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { DndContext } from 'react-dnd';
import { findDOMNode } from 'react-dom';
import throttle from 'lodash.throttle';
import raf from 'raf';
import hoist from 'hoist-non-react-statics';
import { noop, intBetween, getCoords } from './util.js';
import { jsx as _jsx } from "react/jsx-runtime";
var DEFAULT_BUFFER = 150;

var getDisplayName = function getDisplayName(PassedComponent) {
  return PassedComponent.displayName || PassedComponent.name || (typeof PassedComponent === 'string' && PassedComponent.length > 0 ? PassedComponent : 'Unknown');
};

export function createHorizontalStrength(_buffer) {
  return function defaultHorizontalStrength(_ref, point) {
    var x = _ref.x,
        w = _ref.w,
        y = _ref.y,
        h = _ref.h;
    var buffer = Math.min(w / 2, _buffer);
    var inRange = point.x >= x && point.x <= x + w;
    var inBox = inRange && point.y >= y && point.y <= y + h;

    if (inBox) {
      if (point.x < x + buffer) {
        return (point.x - x - buffer) / buffer;
      } else if (point.x > x + w - buffer) {
        return -(x + w - point.x - buffer) / buffer;
      }
    }

    return 0;
  };
}
export function createVerticalStrength(_buffer) {
  return function defaultVerticalStrength(_ref2, point) {
    var y = _ref2.y,
        h = _ref2.h,
        x = _ref2.x,
        w = _ref2.w;
    var buffer = Math.min(h / 2, _buffer);
    var inRange = point.y >= y && point.y <= y + h;
    var inBox = inRange && point.x >= x && point.x <= x + w;

    if (inBox) {
      if (point.y < y + buffer) {
        return (point.y - y - buffer) / buffer;
      } else if (point.y > y + h - buffer) {
        return -(y + h - point.y - buffer) / buffer;
      }
    }

    return 0;
  };
}
export var defaultHorizontalStrength = createHorizontalStrength(DEFAULT_BUFFER);
export var defaultVerticalStrength = createVerticalStrength(DEFAULT_BUFFER);
export default function createScrollingComponent(WrappedComponent) {
  var ScrollingComponent = /*#__PURE__*/function (_Component) {
    _inheritsLoose(ScrollingComponent, _Component);

    function ScrollingComponent(props, ctx) {
      var _this;

      _this = _Component.call(this, props, ctx) || this;

      _this.handleEvent = function (evt) {
        if (_this.dragging && !_this.attached) {
          _this.attach();

          _this.updateScrolling(evt);
        }
      };

      _this.updateScrolling = throttle(function (evt) {
        var _this$container$getBo = _this.container.getBoundingClientRect(),
            x = _this$container$getBo.left,
            y = _this$container$getBo.top,
            w = _this$container$getBo.width,
            h = _this$container$getBo.height;

        var box = {
          x: x,
          y: y,
          w: w,
          h: h
        };
        var coords = getCoords(evt); // calculate strength

        _this.scaleX = _this.props.horizontalStrength(box, coords);
        _this.scaleY = _this.props.verticalStrength(box, coords); // start scrolling if we need to

        if (!_this.frame && (_this.scaleX || _this.scaleY)) {
          _this.startScrolling();
        }
      }, 100, {
        trailing: false
      });
      _this.scaleX = 0;
      _this.scaleY = 0;
      _this.frame = null;
      _this.attached = false;
      _this.dragging = false;
      return _this;
    }

    var _proto = ScrollingComponent.prototype;

    _proto.componentDidMount = function componentDidMount() {
      var _this2 = this;

      this.container = findDOMNode(this.wrappedInstance);
      this.container.addEventListener('dragover', this.handleEvent); // touchmove events don't seem to work across siblings, so we unfortunately
      // have to attach the listeners to the body

      window.document.body.addEventListener('touchmove', this.handleEvent);
      this.clearMonitorSubscription = this.context.dragDropManager.getMonitor().subscribeToStateChange(function () {
        return _this2.handleMonitorChange();
      });
    };

    _proto.componentWillUnmount = function componentWillUnmount() {
      this.container.removeEventListener('dragover', this.handleEvent);
      window.document.body.removeEventListener('touchmove', this.handleEvent);
      this.clearMonitorSubscription();
      this.stopScrolling();
    };

    _proto.handleMonitorChange = function handleMonitorChange() {
      var isDragging = this.context.dragDropManager.getMonitor().isDragging();

      if (!this.dragging && isDragging) {
        this.dragging = true;
      } else if (this.dragging && !isDragging) {
        this.dragging = false;
        this.stopScrolling();
      }
    };

    _proto.attach = function attach() {
      this.attached = true;
      window.document.body.addEventListener('dragover', this.updateScrolling);
      window.document.body.addEventListener('touchmove', this.updateScrolling);
    };

    _proto.detach = function detach() {
      this.attached = false;
      window.document.body.removeEventListener('dragover', this.updateScrolling);
      window.document.body.removeEventListener('touchmove', this.updateScrolling);
    } // Update scaleX and scaleY every 100ms or so
    // and start scrolling if necessary
    ;

    _proto.startScrolling = function startScrolling() {
      var _this3 = this;

      var i = 0;

      var tick = function tick() {
        var scaleX = _this3.scaleX,
            scaleY = _this3.scaleY,
            container = _this3.container;
        var _this3$props = _this3.props,
            strengthMultiplier = _this3$props.strengthMultiplier,
            onScrollChange = _this3$props.onScrollChange; // stop scrolling if there's nothing to do

        if (strengthMultiplier === 0 || scaleX + scaleY === 0) {
          _this3.stopScrolling();

          return;
        } // there's a bug in safari where it seems like we can't get
        // mousemove events from a container that also emits a scroll
        // event that same frame. So we double the strengthMultiplier and only adjust
        // the scroll position at 30fps


        if (i++ % 2) {
          var scrollLeft = container.scrollLeft,
              scrollTop = container.scrollTop,
              scrollWidth = container.scrollWidth,
              scrollHeight = container.scrollHeight,
              clientWidth = container.clientWidth,
              clientHeight = container.clientHeight;
          var newLeft = scaleX ? container.scrollLeft = intBetween(0, scrollWidth - clientWidth, scrollLeft + scaleX * strengthMultiplier) : scrollLeft;
          var newTop = scaleY ? container.scrollTop = intBetween(0, scrollHeight - clientHeight, scrollTop + scaleY * strengthMultiplier) : scrollTop;
          onScrollChange(newLeft, newTop);
        }

        _this3.frame = raf(tick);
      };

      tick();
    };

    _proto.stopScrolling = function stopScrolling() {
      this.detach();
      this.scaleX = 0;
      this.scaleY = 0;

      if (this.frame) {
        raf.cancel(this.frame);
        this.frame = null;
      }
    };

    _proto.render = function render() {
      var _this4 = this;

      var _this$props = this.props,
          strengthMultiplier = _this$props.strengthMultiplier,
          verticalStrength = _this$props.verticalStrength,
          horizontalStrength = _this$props.horizontalStrength,
          onScrollChange = _this$props.onScrollChange,
          props = _objectWithoutPropertiesLoose(_this$props, _excluded);

      return /*#__PURE__*/_jsx(WrappedComponent, _extends({
        ref: function ref(_ref3) {
          _this4.wrappedInstance = _ref3;
        }
      }, props));
    };

    return ScrollingComponent;
  }(Component);

  ScrollingComponent.displayName = "Scrolling(" + getDisplayName(WrappedComponent) + ")";
  ScrollingComponent.propTypes = {
    onScrollChange: PropTypes.func,
    verticalStrength: PropTypes.func,
    horizontalStrength: PropTypes.func,
    strengthMultiplier: PropTypes.number
  };
  ScrollingComponent.defaultProps = {
    onScrollChange: noop,
    verticalStrength: defaultVerticalStrength,
    horizontalStrength: defaultHorizontalStrength,
    strengthMultiplier: 30
  };
  ScrollingComponent.contextType = DndContext;
  return hoist(ScrollingComponent, WrappedComponent);
}